@@ -52,6 +52,7 @@ extern "C"
 
 enum NFAEngineType {
     LIMEX_NFA_32,
+    LIMEX_NFA_64,
     LIMEX_NFA_128,
     LIMEX_NFA_256,
     LIMEX_NFA_384,
@@ -60,15 +61,17 @@ enum NFAEngineType {
     MCCLELLAN_NFA_16,   /**< magic pseudo nfa */
     GOUGH_NFA_8,        /**< magic pseudo nfa */
     GOUGH_NFA_16,       /**< magic pseudo nfa */
-    MPV_NFA_0,          /**< magic pseudo nfa */
-    LBR_NFA_Dot,        /**< magic pseudo nfa */
-    LBR_NFA_Verm,       /**< magic pseudo nfa */
-    LBR_NFA_NVerm,      /**< magic pseudo nfa */
-    LBR_NFA_Shuf,       /**< magic pseudo nfa */
-    LBR_NFA_Truf,       /**< magic pseudo nfa */
-    CASTLE_NFA_0,       /**< magic pseudo nfa */
-    SHENG_NFA_0,        /**< magic pseudo nfa */
-    TAMARAMA_NFA_0,     /**< magic nfa container */
+    MPV_NFA,            /**< magic pseudo nfa */
+    LBR_NFA_DOT,        /**< magic pseudo nfa */
+    LBR_NFA_VERM,       /**< magic pseudo nfa */
+    LBR_NFA_NVERM,      /**< magic pseudo nfa */
+    LBR_NFA_SHUF,       /**< magic pseudo nfa */
+    LBR_NFA_TRUF,       /**< magic pseudo nfa */
+    CASTLE_NFA,         /**< magic pseudo nfa */
+    SHENG_NFA,          /**< magic pseudo nfa */
+    TAMARAMA_NFA,       /**< magic nfa container */
+    MCSHENG_NFA_8,      /**< magic pseudo nfa */
+    MCSHENG_NFA_16,     /**< magic pseudo nfa */
     /** \brief bogus NFA - not used */
     INVALID_NFA
 };
@@ -142,6 +145,12 @@ static really_inline int isMcClellanType(u8 t) {
     return t == MCCLELLAN_NFA_8 || t == MCCLELLAN_NFA_16;
 }
 
+/** \brief True if the given type (from NFA::type) is a Sheng-McClellan hybrid
+ * DFA. */
+static really_inline int isShengMcClellanType(u8 t) {
+    return t == MCSHENG_NFA_8 || t == MCSHENG_NFA_16;
+}
+
 /** \brief True if the given type (from NFA::type) is a Gough DFA. */
 static really_inline int isGoughType(u8 t) {
     return t == GOUGH_NFA_8 || t == GOUGH_NFA_16;
@@ -149,7 +158,7 @@ static really_inline int isGoughType(u8 t) {
 
 /** \brief True if the given type (from NFA::type) is a Sheng DFA. */
 static really_inline int isShengType(u8 t) {
-    return t == SHENG_NFA_0;
+    return t == SHENG_NFA;
 }
 
 /**
@@ -157,13 +166,23 @@ static really_inline int isShengType(u8 t) {
  * Sheng DFA.
  */
 static really_inline int isDfaType(u8 t) {
-    return isMcClellanType(t) || isGoughType(t) || isShengType(t);
+    return isMcClellanType(t) || isGoughType(t) || isShengType(t)
+        || isShengMcClellanType(t);
+}
+
+static really_inline int isBigDfaType(u8 t) {
+    return t == MCCLELLAN_NFA_16 || t == MCSHENG_NFA_16 || t == GOUGH_NFA_16;
+}
+
+static really_inline int isSmallDfaType(u8 t) {
+    return isDfaType(t) && !isBigDfaType(t);
 }
 
 /** \brief True if the given type (from NFA::type) is an NFA. */
 static really_inline int isNfaType(u8 t) {
     switch (t) {
     case LIMEX_NFA_32:
+    case LIMEX_NFA_64:
     case LIMEX_NFA_128:
     case LIMEX_NFA_256:
     case LIMEX_NFA_384:
@@ -178,14 +197,14 @@ static really_inline int isNfaType(u8 t) {
 /** \brief True if the given type (from NFA::type) is an LBR. */
 static really_inline
 int isLbrType(u8 t) {
-    return t == LBR_NFA_Dot || t == LBR_NFA_Verm || t == LBR_NFA_NVerm ||
-           t == LBR_NFA_Shuf || t == LBR_NFA_Truf;
+    return t == LBR_NFA_DOT || t == LBR_NFA_VERM || t == LBR_NFA_NVERM ||
+           t == LBR_NFA_SHUF || t == LBR_NFA_TRUF;
 }
 
 /** \brief True if the given type (from NFA::type) is a container engine. */
 static really_inline
 int isContainerType(u8 t) {
-    return t == TAMARAMA_NFA_0;
+    return t == TAMARAMA_NFA;
 }
 
 static really_inline
@@ -200,14 +219,14 @@ int isMultiTopType(u8 t) {
 /* Use for functions that return an integer. */
 #define NFA_API_NO_IMPL(...)                                                   \
     ({                                                                         \
-        assert(!"not implemented for this engine!");                            \
+        assert(!"not implemented for this engine!");                           \
         0; /* return value, for places that need it */                         \
     })
 
 /* Use for _zombie_status functions. */
 #define NFA_API_ZOMBIE_NO_IMPL(...)                                            \
     ({                                                                         \
-        assert(!"not implemented for this engine!");                            \
+        assert(!"not implemented for this engine!");                           \
         NFA_ZOMBIE_NO;                                                         \
     })
 
