@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2017-2018, Intel Corporation
+ * Copyright (c) 2017-2019, Intel Corporation
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are met:
@@ -2206,6 +2206,14 @@ public:
     ~RoseInstrFlushCombination() override;
 };
 
+class RoseInstrLastFlushCombination
+    : public RoseInstrBaseTrivial<ROSE_INSTR_LAST_FLUSH_COMBINATION,
+                                  ROSE_STRUCT_LAST_FLUSH_COMBINATION,
+                                  RoseInstrLastFlushCombination> {
+public:
+    ~RoseInstrLastFlushCombination() override;
+};
+
 class RoseInstrSetExhaust
     : public RoseInstrBaseNoTargets<ROSE_INSTR_SET_EXHAUST,
                                     ROSE_STRUCT_SET_EXHAUST,
