@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2016-2018, Intel Corporation
+ * Copyright (c) 2016-2019, Intel Corporation
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are met:
@@ -317,6 +317,10 @@ void addFlushCombinationProgram(RoseProgram &program) {
     program.add_before_end(make_unique<RoseInstrFlushCombination>());
 }
 
+void addLastFlushCombinationProgram(RoseProgram &program) {
+    program.add_before_end(make_unique<RoseInstrLastFlushCombination>());
+}
+
 static
 void makeRoleCheckLeftfix(const RoseBuildImpl &build,
                           const map<RoseVertex, left_build_info> &leftfix_info,
