@@ -40,6 +40,7 @@
 #include "nfagraph/ng_som_util.h"
 #include "nfagraph/ng_region.h"
 #include "util/charreach.h"
+#include "util/hash.h"
 #include "util/make_unique.h"
 #include "util/dump_charclass.h"
 #include "util/verify_types.h"
@@ -48,8 +49,6 @@
 #include <deque>
 #include <utility>
 
-#include <boost/functional/hash/hash.hpp>
-
 using namespace std;
 
 namespace ue2 {
@@ -67,13 +66,8 @@ SlotCacheEntry::SlotCacheEntry(const NGHolder &prefix_in,
 size_t SlotEntryHasher::operator()(const SlotCacheEntry &e) const {
     assert(e.prefix);
 
-    using boost::hash_combine;
-
-    size_t v = 0;
-    hash_combine(v, hash_holder(*e.prefix));
-    hash_combine(v, e.parent_slot);
-    hash_combine(v, e.is_reset);
-    hash_combine(v, e.escapes.hash());
+    size_t v = hash_all(hash_holder(*e.prefix), e.parent_slot,
+                        e.is_reset, e.escapes);
 
     DEBUG_PRINTF("%zu vertices, parent_slot=%u, escapes=%s, is_reset=%d "
                  "hashes to %zx\n", num_vertices(*e.prefix), e.parent_slot,
@@ -143,7 +137,7 @@ u32 SomSlotManager::getSomSlot(const NGHolder &prefix,
 
 u32 SomSlotManager::getInitialResetSomSlot(const NGHolder &prefix,
                 const NGHolder &g,
-                const ue2::unordered_map<NFAVertex, u32> &region_map,
+                const unordered_map<NFAVertex, u32> &region_map,
                 u32 last_sent_region, bool *prefix_already_implemented) {
     DEBUG_PRINTF("getting initial reset; last sent region %u\n",
                  last_sent_region);
@@ -171,9 +165,9 @@ u32 SomSlotManager::getInitialResetSomSlot(const NGHolder &prefix,
     // Clone a copy of g (and its region map) that we will be able to store
     // later on.
     shared_ptr<NGHolder> gg = make_shared<NGHolder>();
-    ue2::unordered_map<NFAVertex, NFAVertex> orig_to_copy;
+    unordered_map<NFAVertex, NFAVertex> orig_to_copy;
     cloneHolder(*gg, g, &orig_to_copy);
-    ue2::unordered_map<NFAVertex, u32> gg_region_map;
+    unordered_map<NFAVertex, u32> gg_region_map;
     for (const auto &m : region_map) {
         assert(contains(region_map, m.first));
         gg_region_map.emplace(orig_to_copy.at(m.first), m.second);
