@@ -180,44 +180,52 @@
 #define partial_load_m384(ptr, sz) loadbytes384(ptr, sz)
 #define partial_load_m512(ptr, sz) loadbytes512(ptr, sz)
 
-#define store_compressed_u32(ptr, x, m)     storecompressed32(ptr, x, m)
-#define store_compressed_u64a(ptr, x, m)    storecompressed64(ptr, x, m)
-#define store_compressed_m128(ptr, x, m)    storecompressed128(ptr, x, m)
-#define store_compressed_m256(ptr, x, m)    storecompressed256(ptr, x, m)
-#define store_compressed_m384(ptr, x, m)    storecompressed384(ptr, x, m)
-#define store_compressed_m512(ptr, x, m)    storecompressed512(ptr, x, m)
+#define store_compressed_u32(ptr, x, m, len)  storecompressed32(ptr, x, m, len)
+#define store_compressed_u64a(ptr, x, m, len) storecompressed64(ptr, x, m, len)
+#define store_compressed_m128(ptr, x, m, len) storecompressed128(ptr, x, m, len)
+#define store_compressed_m256(ptr, x, m, len) storecompressed256(ptr, x, m, len)
+#define store_compressed_m384(ptr, x, m, len) storecompressed384(ptr, x, m, len)
+#define store_compressed_m512(ptr, x, m, len) storecompressed512(ptr, x, m, len)
 
-#define load_compressed_u32(x, ptr, m)      loadcompressed32(x, ptr, m)
-#define load_compressed_u64a(x, ptr, m)     loadcompressed64(x, ptr, m)
-#define load_compressed_m128(x, ptr, m)     loadcompressed128(x, ptr, m)
-#define load_compressed_m256(x, ptr, m)     loadcompressed256(x, ptr, m)
-#define load_compressed_m384(x, ptr, m)     loadcompressed384(x, ptr, m)
-#define load_compressed_m512(x, ptr, m)     loadcompressed512(x, ptr, m)
+#define load_compressed_u32(x, ptr, m, len)   loadcompressed32(x, ptr, m, len)
+#define load_compressed_u64a(x, ptr, m, len)  loadcompressed64(x, ptr, m, len)
+#define load_compressed_m128(x, ptr, m, len)  loadcompressed128(x, ptr, m, len)
+#define load_compressed_m256(x, ptr, m, len)  loadcompressed256(x, ptr, m, len)
+#define load_compressed_m384(x, ptr, m, len)  loadcompressed384(x, ptr, m, len)
+#define load_compressed_m512(x, ptr, m, len)  loadcompressed512(x, ptr, m, len)
 
-static really_inline void clearbit_u32(u32 *p, u32 n) {
+static really_inline
+void clearbit_u32(u32 *p, u32 n) {
     assert(n < sizeof(*p) * 8);
     *p &= ~(1U << n);
 }
-static really_inline void clearbit_u64a(u64a *p, u32 n) {
+
+static really_inline
+void clearbit_u64a(u64a *p, u32 n) {
     assert(n < sizeof(*p) * 8);
     *p &= ~(1ULL << n);
 }
+
 #define clearbit_m128(ptr, n)   (clearbit128(ptr, n))
 #define clearbit_m256(ptr, n)   (clearbit256(ptr, n))
 #define clearbit_m384(ptr, n)   (clearbit384(ptr, n))
 #define clearbit_m512(ptr, n)   (clearbit512(ptr, n))
 
-static really_inline char testbit_u32(const u32 *p, u32 n) {
-    assert(n < sizeof(*p) * 8);
-    return !!(*p & (1U << n));
+static really_inline
+char testbit_u32(u32 val, u32 n) {
+    assert(n < sizeof(val) * 8);
+    return !!(val & (1U << n));
 }
-static really_inline char testbit_u64a(const u64a *p, u32 n) {
-    assert(n < sizeof(*p) * 8);
-    return !!(*p & (1ULL << n));
+
+static really_inline
+char testbit_u64a(u64a val, u32 n) {
+    assert(n < sizeof(val) * 8);
+    return !!(val & (1ULL << n));
 }
-#define testbit_m128(ptr, n)    (testbit128(ptr, n))
-#define testbit_m256(ptr, n)    (testbit256(ptr, n))
-#define testbit_m384(ptr, n)    (testbit384(ptr, n))
-#define testbit_m512(ptr, n)    (testbit512(ptr, n))
+
+#define testbit_m128(val, n)    (testbit128(val, n))
+#define testbit_m256(val, n)    (testbit256(val, n))
+#define testbit_m384(val, n)    (testbit384(val, n))
+#define testbit_m512(val, n)    (testbit512(val, n))
 
 #endif
